==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2302_169.zip: ronportal/ron/controls/combobox.php <==
<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace Ron\Controls;

use Ron\ArrayUtils;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Tracy\Debugger;


class ComboBox extends Options implements ViewObject
{
    /** @var string     jmeno datoveho pole objektu, ktere obsahuje title. V load se nastavuje na 'title' */
    public $titleFieldName;
    /** @var string     jmeno datoveho pole objektu, ktere obsahuje class. V load se nastavuje na 'cssClass' */
    public $cssClassFieldName;

    /** @var Option $selectedOption */
    public $selectedOption;

    /** @var int        Indexy jednotlivych poli v SQL dotazu */
    public $keyIndex;
    public $captionIndex;
    public $titleIndex;
    public $cssClassIndex;

    /** @var string     Prefix pro preklad retezcu - pokud je zadan, tak se texty nactene z databaze pokusi prelozit */
    public $translationPrefix;
    /** @var string     Prefix pro preklad titulku */
    public $titleTranslationPrefix;
    /** @var int        Index pole, ktere se pouzije jako kod pro preklad */
    public $translationValueIndex;

    /** @var array      Asociativni pole indexu dalsich atributu, ktere se pridaji k polozkam (nazev atributu => index) */
    public $attrIndexes;

    /** @var bool $alwaysShowSelected    pokud je true, zobrazi se zadana hodnota, i kdyz neni nactena z DB */
    public $alwaysShowSelected;

    /** @var string $varName    Jmeno promenne v HTML */
    public $varName;
    public $className;

    /** @var bool $noHtmlConvert    priznak, ze se nema provest htmlTextConvert s popisem (defaultne = false) */
    public $noHtmlConvert;

    /** @var string $emptyRow   hodnota, ktera se ma zobrazit v prvnim (prazdnem) radku - pokud je null, nezobrazuje se */
    public $emptyRow;

    /** @var bool $showCode     Priznak, ze se ma zobrazovat i kod (v zavorce) */
    public $showCode;

    /** @var bool $dataTypeComparison      Priznak, ze se maji hodnoty porovnavat vcetne datoveho typu, default = true */
    public $dataTypeComparison;

    /** @var array|string       Asociativni pole HTML parametru, pripadne muze byt string */
    public $htmlParams;

    public $readOnly;

    /** @var mixed[]            Pole "preferovanych" hodnot. Pri nacitani z databaze se zobrazi na zacatku */
    public $preferredValues = [];


    /**
     * @param string $varName
     * @param Option[]|object[]|string[] [$data]
     * @param string $keyFieldName
     * @param string $valueFieldName
     */
    function __construct($varName, $data = null, $keyFieldName = 'key', $valueFieldName = 'caption')
    {
        parent::__construct($data, $keyFieldName, $valueFieldName);

        $this->varName = $varName;

        $this ->keyIndex = 0;
        $this ->captionIndex = 1;

        $this ->dataTypeComparison = true;

        $this ->htmlParams = ['class' => 'form-control'];
        $this ->readOnly = false;
    }

    /**
     * @param int $keyIndex
     * @return $this
     */
    public function setKeyIndex($keyIndex)
    {
        $this->keyIndex = $keyIndex;
        return $this;
    }

    /**
     * @param int $captionIndex
     * @return $this
     */
    public function setCaptionIndex($captionIndex)
    {
        $this->captionIndex = $captionIndex;
        return $this;
    }

    /**
     * @param mixed $titleIndex
     * @return $this
     */
    public function setTitleIndex($titleIndex)
    {
        $this->titleIndex = $titleIndex;
        return $this;
    }

    /**
     * @param mixed $cssClassIndex
     * @return $this
     */
    public function setCssClassIndex($cssClassIndex)
    {
        $this->cssClassIndex = $cssClassIndex;
        return $this;
    }


    /**
     * @param array $attrIndexes
     * @return $this
     */
    public function setAttrIndexes($attrIndexes)
    {
        $this->attrIndexes = $attrIndexes;
        return $this;
    }




    /**
     * @param mixed $className
     * @return $this
     */
    public function setClassName($className)
    {
        $this->className = $className;
        return $this;
    }

    /**
     * @param string $translationPrefix
     * @param int $translationValueIndex
     * @return $this
     */
    public function setTranslationPrefix($translationPrefix, $translationValueIndex = 0)
    {
        $this->translationPrefix = $translationPrefix;
        $this->translationValueIndex = $translationValueIndex;
        return $this;
    }


    /**
     * @param string $emptyRow
     * @return $this
     */
    public function setEmptyRow($emptyRow)
    {
        $this->emptyRow = $emptyRow;
        return $this;
    }

    /**
     * @param boolean $showCode
     * @return $this
     */
    public function setShowCode($showCode)
    {
        $this->showCode = $showCode;
        return $this;
    }

    /**
     * @param boolean $dataTypeComparison
     * @return $this
     */
    public function setDataTypeComparison($dataTypeComparison)
    {
        $this->dataTypeComparison = $dataTypeComparison;
        return $this;
    }

    /**
     * @param string[] $htmlParams
     * @return $this
     */
    public function setHtmlParams($htmlParams)
    {
        $this->htmlParams = $htmlParams;
        return $this;
    }

    /**
     * @param string[] $htmlParams
     * @return $this
     */
    public function addHtmlParams($htmlParams)
    {
        if (!$this ->htmlParams) {
            $this ->htmlParams = $htmlParams;
        } else {
            $this ->htmlParams = array_merge($this ->htmlParams, $htmlParams);
        }

        return $this;
    }

    /**
     * @param string $htmlParamsName
     * @param string $value
     * @return $this
     */
    public function addHtmlParam($htmlParamsName, $value)
    {
        if (!$this ->htmlParams) {
            $this ->htmlParams = [];
        }

        $this ->htmlParams[$htmlParamsName] = $value;

        return $this;
    }

    /**
     * @param boolean $alwaysShowSelected
     * @return $this
     */
    public function setAlwaysShowSelected($alwaysShowSelected)
    {
        $this->alwaysShowSelected = $alwaysShowSelected;
        return $this;
    }



    function setSelectedOption($selectedValue)
    {
        if ($selectedValue instanceof Option) {
            $this ->selectedOption = $selectedValue;
            return;
        }

        $selectedOption = isset($selectedValue) ? $this->findOption($selectedValue, $this ->dataTypeComparison) : null;

        if (!$selectedOption  &&  isset($selectedValue)) {
            if ($this ->alwaysShowSelected) {
                $selectedOption = new Option($selectedValue, $selectedValue);
                //$this ->data[] = $selectedOption;
            } else if (!isset($this ->emptyRow)  &&  sizeof($this ->data)) { // pokud je empty row, tak by nemela byt vybrana zadna jina
                $selectedOption = $this ->data[ArrayUtils::firstKey($this ->data)];
            }
        }

        $this ->selectedOption = $selectedOption;
    }


    /**
     * @param Database $database
     * @param string|SqlStatement $sql
     * @param mixed $selectedValue
     * @param Parameters $sqlParams
     * @param callable $filterFunction Funkce, ktera slouzi pro odfiltrovani nactenych radku. Pokud je zadana, tak se polozka do seznamu prida jen v pripade, ze funkce vrati true
     * @return $this
     * @throws DBException
     */
    function loadData($database, $sql, $selectedValue, $sqlParams = null, $filterFunction = null)
    {
        if ($this instanceof ICreateItem) {
            /** @noinspection PhpUndefinedMethodInspection */
            /** @noinspection PhpUndefinedFieldInspection */
            $this ->loadSQLCustomCreate($database, $sql, $sqlParams, $this, $this ->emptyRow);
        } else {
            $this->titleFieldName = 'title';
            $this->cssClassFieldName = 'cssClass';
//            $this->loadSQL($database, $sql, $sqlParams, $this->keyIndex, $this->captionIndex, $this->titleIndex, $this->cssClassIndex, $this ->attrIndexes, $this ->preferredValues, $filterFunction);
//            $this->setSelectedOption($selectedValue);

            $newData = (new OptionLoader($database))
                ->setSql($sql, $sqlParams)
                ->setKeyIndex($this->keyIndex)
                ->setCaptionIndex($this->captionIndex)
                ->setTitleIndex($this->titleIndex)
                ->setCssClassIndex($this->cssClassIndex)
                ->setAttrIndexes($this ->attrIndexes)
                ->setPreferredValues($this ->preferredValues)
                ->setFilterFunction($filterFunction)
                ->setTranslationPrefix($this ->translationPrefix, $this ->translationValueIndex)
                ->setTitleTranslationPrefix($this ->titleTranslationPrefix)
                ->loadData();

            $this ->addOptions($newData);

            $this->setSelectedOption($selectedValue);
        }

        return $this;
    }


    /**
     * Vrati nactena data pro JSON
     */
    function getJsonData()
    {
        $result = [];
        foreach ($this ->data as $option) {
            $result[] = $option ->getJson();
        }

        return $result;
    }


    /**
     * @param string[] $params  Asociativni pole HTML atributu
     */
    function renderHtmlParams($params)
    {
        $varName = $this ->varName ?: ($params['name'] ?? null);
        if ($varName) {
            echo " name='{$varName}'";
        }

        $className = $this ->className ?? $params['class'] ?? null;
        $errorClass = $varName ? MessageManager::getClass($varName) : null;
        if ($className || $errorClass) {
            $className = ($className ?? '') . ($className ? ' ' : '') . ($errorClass ?? '');
            echo " class='{$className}'";
        }

        foreach ($params as $name => $value) {
            switch ($name) {
                case 'name' :
                case 'class' :
                    break;

                default :
                    echo " $name='" . StringUtils::htmlTextConvert($value) . "'";
            }
        }

        if ($varName  &&  !isset($params['title'])) {
            $titleStr = MessageManager::getTitleHtml($varName);
            if ($titleStr) {
                echo ' ' . $titleStr;
            }
        }
    }


    function render()
    {
        if ($this ->readOnly) {
            if ($this ->selectedOption) {
                if ($this ->titleFieldName  &&  $this ->selectedOption ->{$this ->titleFieldName}) {
                    echo '<span title="' . StringUtils::htmlTextConvert($this ->selectedOption ->{$this ->titleFieldName}) . '">';
                }

                echo $this ->noHtmlConvert
                    ? $this->selectedOption->{$this ->valueFieldName}
                    : StringUtils::htmlTextConvert($this->selectedOption->{$this ->valueFieldName});

                if ($this ->titleFieldName  &&  $this ->selectedOption ->{$this ->titleFieldName}) {
                    echo '</span>';
                }
            }
        } else {
            echo '<select';
            if (is_array($this->htmlParams)) {
                $this ->renderHtmlParams($this ->htmlParams);
            } else {
                echo
                    ($this->varName ?' name="' . $this->varName . '"' : '') .
                    ($this->className ? ' class="' . $this->className . '"' : '') .
                    ($this->htmlParams ? ' ' . $this->htmlParams : '');
            }
            echo '>';

            $first = true;

            // empty row
            if (isset($this ->emptyRow)) {
                $emptyRowText = $this ->emptyRow === true
                    ? '&nbsp;'
                    :  $this ->emptyRow;

                echo "<option value=''>$emptyRowText</option>";

                $first = false;
            }

            // zapise jednotlive polozky
            $selectedOptionsRendered = false;

            foreach ($this->data as $i => $option) {
                echo
                    '<option ' .
                    ' value="' . StringUtils::htmlTextConvert($option->{$this ->keyFieldName}) . '"';

                if (($first && !isset($this->selectedOption)) || ($this->selectedOption === $option)) {
                    echo ' selected="selected"';
                }

                $cssClass = ($this ->cssClassFieldName  &&  $option->{$this ->cssClassFieldName})
                    ? $option->{$this ->cssClassFieldName}
                    : '';

                if ($option ->cssClass) {
                    $cssClass .= ($cssClass ? ' ' : '') . $option ->cssClass;
                }
                if ($cssClass) {
                    echo ' class="' . StringUtils::htmlTextConvert($cssClass) . '"';
                }
                if ($this ->titleFieldName  &&  $option->{$this ->titleFieldName}) {
                    echo ' title="' . StringUtils::htmlTextConvert($option->{$this ->titleFieldName}) . '"';
                }

                if ($option ->params) {
                    foreach ($option ->params as $paramName => $paramValue) {
                        echo ' ' . $paramName . '="' . StringUtils::htmlTextConvert($paramValue) . '"';
                    }
                }


                echo '>';

                echo $this->noHtmlConvert
                    ? $option->{$this ->valueFieldName}
                    : StringUtils::htmlTextConvert($option->{$this ->valueFieldName});

                if ($this->showCode && $option->{$this ->keyFieldName} !== $option->{$this ->valueFieldName}  &&  $option ->{$this ->keyFieldName} !== '') {
                    echo ' (' . StringUtils::htmlTextConvert($option->{$this ->keyFieldName}) . ')';
                }

                echo '</option>';

                if ($this->selectedOption === $option) {
                    $selectedOptionsRendered = true;
                }

                $first = false;
            }

            if ($this ->selectedOption  &&  !$selectedOptionsRendered  &&  $this ->alwaysShowSelected) {
                echo
                    '<option value="' . StringUtils::htmlTextConvert($this ->selectedOption ->{$this ->keyFieldName}) . '" selected="selected">' .
                    StringUtils::htmlTextConvert($this ->selectedOption ->{$this ->valueFieldName}) .
                    '</option>';
            }

            echo "</select>\n";
        }
    }


    static function renderComboFromArray($data, $varName, $selectedValue, $htmlParams = [], $dataTypeComparison = true)
    {
        if (!$data) return;

        $combo = new ComboBox($varName, $data);
        $combo ->dataTypeComparison = $dataTypeComparison;
        $combo ->htmlParams = $htmlParams;

        if (isset($selectedValue)) {
            $combo ->selectedOption = $combo ->findOption($selectedValue, $dataTypeComparison);
        }

        $combo ->render();
    }


    /**
     * @param string $varName
     * @param Option[]|object[] $options    Seznam polozek pro zobrazeni
     * @param mixed     $selectedValue      Vybrana hodnota
     * @param string[]|null  $comboBoxProps     Asociativni pole vlastnosti comboboxu
     */
    public static function renderComboBox($varName, $options, $selectedValue = null, $comboBoxProps = null)
    {
        $comboBox = new static($varName, $options);
        if ($comboBoxProps) {
            foreach ($comboBoxProps as $propName => $propValue) {
                $comboBox ->{$propName} = $propValue;
            }
        }

        if (isset($selectedValue)) {
            $comboBox ->setSelectedOption($selectedValue);
        }

        $comboBox ->render();
    }
}

==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2311_175.zip: ronportal/ron/controls/combobox.php <==
<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace Ron\Controls;

use Ron\ArrayUtils;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Tracy\Debugger;


class ComboBox extends Options implements ViewObject
{
    /** @var string     jmeno datoveho pole objektu, ktere obsahuje title. V load se nastavuje na 'title' */
    public $titleFieldName;
    /** @var string     jmeno datoveho pole objektu, ktere obsahuje class. V load se nastavuje na 'cssClass' */
    public $cssClassFieldName;

    /** @var Option $selectedOption */
    public $selectedOption;

    /** @var int        Indexy jednotlivych poli v SQL dotazu */
    public $keyIndex;
    public $captionIndex;
    public $titleIndex;
    public $cssClassIndex;

    /** @var string     Prefix pro preklad retezcu - pokud je zadan, tak se texty nactene z databaze pokusi prelozit */
    public $translationPrefix;
    /** @var string     Prefix pro preklad titulku */
    public $titleTranslationPrefix;
    /** @var int        Index pole, ktere se pouzije jako kod pro preklad */
    public $translationValueIndex;

    /** @var array      Asociativni pole indexu dalsich atributu, ktere se pridaji k polozkam (nazev atributu => index) */
    public $attrIndexes;

    /** @var bool $alwaysShowSelected    pokud je true, zobrazi se zadana hodnota, i kdyz neni nactena z DB */
    public $alwaysShowSelected;

    /** @var string $varName    Jmeno promenne v HTML */
    public $varName;
    public $className;

    /** @var bool $noHtmlConvert    priznak, ze se nema provest htmlTextConvert s popisem (defaultne = false) */
    public $noHtmlConvert;

    /** @var string $emptyRow   hodnota, ktera se ma zobrazit v prvnim (prazdnem) radku - pokud je null, nezobrazuje se */
    public $emptyRow;

    /** @var bool $showCode     Priznak, ze se ma zobrazovat i kod (v zavorce) */
    public $showCode;

    /** @var bool $dataTypeComparison      Priznak, ze se maji hodnoty porovnavat vcetne datoveho typu, default = true */
    public $dataTypeComparison;

    /** @var array|string       Asociativni pole HTML parametru, pripadne muze byt string */
    public $htmlParams;

    public $readOnly;

    /** @var mixed[]            Pole "preferovanych" hodnot. Pri nacitani z databaze se zobrazi na zacatku */
    public $preferredValues = [];


    /**
     * @param string $varName
     * @param Option[]|object[]|string[] [$data]
     * @param string $keyFieldName
     * @param string $valueFieldName
     */
    function __construct($varName, $data = null, $keyFieldName = 'key', $valueFieldName = 'caption')
    {
        parent::__construct($data, $keyFieldName, $valueFieldName);

        $this->varName = $varName;

        $this ->keyIndex = 0;
        $this ->captionIndex = 1;

        $this ->dataTypeComparison = true;

        $this ->htmlParams = ['class' => 'form-control'];
        $this ->readOnly = false;
    }

    /**
     * @param int $keyIndex
     * @return $this
     */
    public function setKeyIndex($keyIndex)
    {
        $this->keyIndex = $keyIndex;
        return $this;
    }

    /**
     * @param int $captionIndex
     * @return $this
     */
    public function setCaptionIndex($captionIndex)
    {
        $this->captionIndex = $captionIndex;
        return $this;
    }

    /**
     * @param mixed $titleIndex
     * @return $this
     */
    public function setTitleIndex($titleIndex)
    {
        $this->titleIndex = $titleIndex;
        return $this;
    }

    /**
     * @param mixed $cssClassIndex
     * @return $this
     */
    public function setCssClassIndex($cssClassIndex)
    {
        $this->cssClassIndex = $cssClassIndex;
        return $this;
    }


    /**
     * @param array $attrIndexes
     * @return $this
     */
    public function setAttrIndexes($attrIndexes)
    {
        $this->attrIndexes = $attrIndexes;
        return $this;
    }




    /**
     * @param mixed $className
     * @return $this
     */
    public function setClassName($className)
    {
        $this->className = $className;
        return $this;
    }

    /**
     * @param string $translationPrefix
     * @param int $translationValueIndex
     * @return $this
     */
    public function setTranslationPrefix($translationPrefix, $translationValueIndex = 0)
    {
        $this->translationPrefix = $translationPrefix;
        $this->translationValueIndex = $translationValueIndex;
        return $this;
    }


    /**
     * @param string $emptyRow
     * @return $this
     */
    public function setEmptyRow($emptyRow)
    {
        $this->emptyRow = $emptyRow;
        return $this;
    }

    /**
     * @param boolean $showCode
     * @return $this
     */
    public function setShowCode($showCode)
    {
        $this->showCode = $showCode;
        return $this;
    }

    /**
     * @param boolean $dataTypeComparison
     * @return $this
     */
    public function setDataTypeComparison($dataTypeComparison)
    {
        $this->dataTypeComparison = $dataTypeComparison;
        return $this;
    }

    /**
     * @param string[] $htmlParams
     * @return $this
     */
    public function setHtmlParams($htmlParams)
    {
        $this->htmlParams = $htmlParams;
        return $this;
    }

    /**
     * @param string[] $htmlParams
     * @return $this
     */
    public function addHtmlParams($htmlParams)
    {
        if (!$this ->htmlParams) {
            $this ->htmlParams = $htmlParams;
        } else {
            $this ->htmlParams = array_merge($this ->htmlParams, $htmlParams);
        }

        return $this;
    }

    /**
     * @param string $htmlParamsName
     * @param string $value
     * @return $this
     */
    public function addHtmlParam($htmlParamsName, $value)
    {
        if (!$this ->htmlParams) {
            $this ->htmlParams = [];
        }

        $this ->htmlParams[$htmlParamsName] = $value;

        return $this;
    }

    /**
     * @param boolean $alwaysShowSelected
     * @return $this
     */
    public function setAlwaysShowSelected($alwaysShowSelected)
    {
        $this->alwaysShowSelected = $alwaysShowSelected;
        return $this;
    }



    function setSelectedOption($selectedValue)
    {
        if ($selectedValue instanceof Option) {
            $this ->selectedOption = $selectedValue;
            return;
        }

        $selectedOption = isset($selectedValue) ? $this->findOption($selectedValue, $this ->dataTypeComparison) : null;

        if (!$selectedOption  &&  isset($selectedValue)) {
            if ($this ->alwaysShowSelected) {
                $selectedOption = new Option($selectedValue, $selectedValue);
                //$this ->data[] = $selectedOption;
            } else if (!isset($this ->emptyRow)  &&  sizeof($this ->data)) { // pokud je empty row, tak by nemela byt vybrana zadna jina
                $selectedOption = $this ->data[ArrayUtils::firstKey($this ->data)];
            }
        }

        $this ->selectedOption = $selectedOption;
    }


    /**
     * @param Database $database
     * @param string|SqlStatement $sql
     * @param mixed $selectedValue
     * @param Parameters $sqlParams
     * @param callable $filterFunction Funkce, ktera slouzi pro odfiltrovani nactenych radku. Pokud je zadana, tak se polozka do seznamu prida jen v pripade, ze funkce vrati true
     * @return $this
     * @throws DBException
     */
    function loadData($database, $sql, $selectedValue, $sqlParams = null, $filterFunction = null)
    {
        if ($this instanceof ICreateItem) {
            $this ->loadSQLCustomCreate($database, $sql, $sqlParams, $this);
        } else {
            $this->titleFieldName = 'title';
            $this->cssClassFieldName = 'cssClass';
//            $this->loadSQL($database, $sql, $sqlParams, $this->keyIndex, $this->captionIndex, $this->titleIndex, $this->cssClassIndex, $this ->attrIndexes, $this ->preferredValues, $filterFunction);
//            $this->setSelectedOption($selectedValue);

            $newData = (new OptionLoader($database))
                ->setSql($sql, $sqlParams)
                ->setKeyIndex($this->keyIndex)
                ->setCaptionIndex($this->captionIndex)
                ->setTitleIndex($this->titleIndex)
                ->setCssClassIndex($this->cssClassIndex)
                ->setAttrIndexes($this ->attrIndexes)
                ->setPreferredValues($this ->preferredValues)
                ->setFilterFunction($filterFunction)
                ->setTranslationPrefix($this ->translationPrefix, $this ->translationValueIndex)
                ->setTitleTranslationPrefix($this ->titleTranslationPrefix)
                ->loadData();

            $this ->addOptions($newData);

            $this->setSelectedOption($selectedValue);
        }

        return $this;
    }


    /**
     * Vrati nactena data pro JSON
     */
    function getJsonData()
    {
        $result = [];
        foreach ($this ->data as $option) {
            $result[] = $option ->getJson();
        }

        return $result;
    }


    /**
     * @param string[] $params  Asociativni pole HTML atributu
     */
    function renderHtmlParams($params)
    {
        $varName = $this ->varName ?: ($params['name'] ?? null);
        if ($varName) {
            echo " name='{$varName}'";
        }

        $className = $this ->className ?? $params['class'] ?? null;
        $errorClass = $varName ? MessageManager::getClass($varName) : null;
        if ($className || $errorClass) {
            $className = ($className ?? '') . ($className ? ' ' : '') . ($errorClass ?? '');
            echo " class='{$className}'";
        }

        foreach ($params as $name => $value) {
            switch ($name) {
                case 'name' :
                case 'class' :
                    break;

                default :
                    echo " $name='" . StringUtils::htmlTextConvert($value) . "'";
            }
        }

        if ($varName  &&  !isset($params['title'])) {
            $titleStr = MessageManager::getTitleHtml($varName);
            if ($titleStr) {
                echo ' ' . $titleStr;
            }
        }
    }


    function render()
    {
        if ($this ->readOnly) {
            if ($this ->selectedOption) {
                if ($this ->titleFieldName  &&  $this ->selectedOption ->{$this ->titleFieldName}) {
                    echo '<span title="' . StringUtils::htmlTextConvert($this ->selectedOption ->{$this ->titleFieldName}) . '">';
                }

                echo $this ->noHtmlConvert
                    ? $this->selectedOption->{$this ->valueFieldName}
                    : StringUtils::htmlTextConvert($this->selectedOption->{$this ->valueFieldName});

                if ($this ->titleFieldName  &&  $this ->selectedOption ->{$this ->titleFieldName}) {
                    echo '</span>';
                }
            }
        } else {
            echo '<select';
            if (is_array($this->htmlParams)) {
                $this ->renderHtmlParams($this ->htmlParams);
            } else {
                echo
                    ($this->varName ?' name="' . $this->varName . '"' : '') .
                    ($this->className ? ' class="' . $this->className . '"' : '') .
                    ($this->htmlParams ? ' ' . $this->htmlParams : '');
            }
            echo '>';

            $first = true;

            // empty row
            if (isset($this ->emptyRow)) {
                $emptyRowText = $this ->emptyRow === true
                    ? '&nbsp;'
                    :  $this ->emptyRow;

                echo "<option value=''>$emptyRowText</option>";

                $first = false;
            }

            // zapise jednotlive polozky
            $selectedOptionsRendered = false;

            foreach ($this->data as $i => $option) {
                echo
                    '<option ' .
                    ' value="' . StringUtils::htmlTextConvert($option->{$this ->keyFieldName}) . '"';

                if (($first && !isset($this->selectedOption)) || ($this->selectedOption === $option)) {
                    echo ' selected="selected"';
                }

                $cssClass = ($this ->cssClassFieldName  &&  $option->{$this ->cssClassFieldName})
                    ? $option->{$this ->cssClassFieldName}
                    : '';

                if ($option ->cssClass) {
                    $cssClass .= ($cssClass ? ' ' : '') . $option ->cssClass;
                }
                if ($cssClass) {
                    echo ' class="' . StringUtils::htmlTextConvert($cssClass) . '"';
                }
                if ($this ->titleFieldName  &&  $option->{$this ->titleFieldName}) {
                    echo ' title="' . StringUtils::htmlTextConvert($option->{$this ->titleFieldName}) . '"';
                }

                if ($option ->params) {
                    foreach ($option ->params as $paramName => $paramValue) {
                        echo ' ' . $paramName . '="' . StringUtils::htmlTextConvert($paramValue) . '"';
                    }
                }


                echo '>';

                echo $this->noHtmlConvert
                    ? $option->{$this ->valueFieldName}
                    : StringUtils::htmlTextConvert($option->{$this ->valueFieldName});

                if ($this->showCode && $option->{$this ->keyFieldName} !== $option->{$this ->valueFieldName}  &&  $option ->{$this ->keyFieldName} !== '') {
                    echo ' (' . StringUtils::htmlTextConvert($option->{$this ->keyFieldName}) . ')';
                }

                echo '</option>';

                if ($this->selectedOption === $option) {
                    $selectedOptionsRendered = true;
                }

                $first = false;
            }

            if ($this ->selectedOption  &&  !$selectedOptionsRendered  &&  $this ->alwaysShowSelected) {
                echo
                    '<option value="' . StringUtils::htmlTextConvert($this ->selectedOption ->{$this ->keyFieldName}) . '" selected="selected">' .
                    StringUtils::htmlTextConvert($this ->selectedOption ->{$this ->valueFieldName}) .
                    '</option>';
            }

            echo "</select>\n";
        }
    }


    static function renderComboFromArray($data, $varName, $selectedValue, $htmlParams = [], $dataTypeComparison = true)
    {
        if (!$data) return;

        $combo = new ComboBox($varName, $data);
        $combo ->dataTypeComparison = $dataTypeComparison;
        $combo ->htmlParams = $htmlParams;

        if (isset($selectedValue)) {
            $combo ->selectedOption = $combo ->findOption($selectedValue, $dataTypeComparison);
        }

        $combo ->render();
    }


    /**
     * @param string $varName
     * @param Option[]|object[] $options    Seznam polozek pro zobrazeni
     * @param mixed     $selectedValue      Vybrana hodnota
     * @param string[]|null  $comboBoxProps     Asociativni pole vlastnosti comboboxu
     */
    public static function renderComboBox($varName, $options, $selectedValue = null, $comboBoxProps = null)
    {
        $comboBox = new static($varName, $options);
        if ($comboBoxProps) {
            foreach ($comboBoxProps as $propName => $propValue) {
                $comboBox ->{$propName} = $propValue;
            }
        }

        if (isset($selectedValue)) {
            $comboBox ->setSelectedOption($selectedValue);
        }

        $comboBox ->render();
    }
}
